@@ -15,6 +15,8 @@ local func_type = eolian.function_type
 local obj_scope = eolian.object_scope
 local param_dir = eolian.parameter_dir
 
+local gen_unit
+
 cutil.init_module(function()
     dom = log.Domain("lualian")
     if not dom:is_valid() then
@@ -55,6 +57,9 @@ local num_others = {
 }
 
 local is_num = function(x)
+    if not x then
+        return false
+    end
     if num_others [x       ] then return true end
     if int_builtin[x       ] then return true end
     if int_builtin["u" .. x] then return true end
@@ -623,14 +628,16 @@ local gen_contents = function(klass)
     -- first try properties
     local props = klass:functions_get(func_type.PROPERTY):to_array()
     for i, v in ipairs(props) do
-        if v:scope_get() == obj_scope.PUBLIC and not v:is_c_only() then
+        local gscope = v:scope_get(func_type.PROP_GET)
+        local sscope = v:scope_get(func_type.PROP_SET)
+        if (gscope == obj_scope.PUBLIC or sscope == obj_scope.PUBLIC) and not v:is_c_only() then
             local ftype  = v:type_get()
             local fread  = (ftype == func_type.PROPERTY or ftype == func_type.PROP_GET)
             local fwrite = (ftype == func_type.PROPERTY or ftype == func_type.PROP_SET)
-            if fwrite then
+            if fwrite and sscope == obj_scope.PUBLIC then
                 cnt[#cnt + 1] = Property(v, func_type.PROP_SET)
             end
-            if fread then
+            if fread and gscope == obj_scope.PUBLIC then
                 cnt[#cnt + 1] = Property(v, func_type.PROP_GET)
             end
         end
@@ -638,7 +645,7 @@ local gen_contents = function(klass)
     -- then methods
     local meths = klass:functions_get(func_type.METHOD):to_array()
     for i, v in ipairs(meths) do
-        if v:scope_get() == obj_scope.PUBLIC and not v:is_c_only() then
+        if v:scope_get(func_type.METHOD) == obj_scope.PUBLIC and not v:is_c_only() then
             cnt[#cnt + 1] = Method(v)
         end
     end
@@ -664,7 +671,7 @@ local gen_class = function(klass)
     local mixins  = {} -- also includes ifaces, they're separated later
     for i = 1, #inherits do
         local v = inherits[i]
-        local tp = eolian.class_get_by_name(v):type_get()
+        local tp = eolian.class_get_by_name(gen_unit, v):type_get()
         if tp == class_type.REGULAR or tp == class_type.ABSTRACT then
             parents[#parents + 1] = v
         elseif tp == class_type.INTERFACE or tp == class_type.MIXIN then
@@ -691,14 +698,16 @@ M.system_directory_scan = function()
 end
 
 M.generate = function(fname, fstream)
-    if not eolian.file_parse(fname) then
+    local unit = eolian.file_parse(fname)
+    if unit == nil then
         error("Failed parsing file: " .. fname)
     end
+    gen_unit = unit
     if not eolian.database_validate() then
         error("Failed validating database.")
     end
     local sfn = fname:match(".*[\\/](.+)$") or fname
-    local klass = eolian.class_get_by_file(sfn)
+    local klass = eolian.class_get_by_file(unit, sfn)
     File(fname, klass, { gen_class(klass) }):generate(fstream or io.stdout)
 end
 
